# -*- coding: utf-8 -*-

"""
Project name: Open Methodology for Security Tool Developers
Project URL: https://github.com/cr0hn/OMSTD

Copyright (c) 2014, cr0hn<-AT->cr0hn.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

__author__ = 'cr0hn - cr0hn<-at->cr0hn.com (@ggdaniel)'

import argparse


# ----------------------------------------------------------------------
def main():
    from .api import Parameters, run_console, PASSWORD_MD5_CRACKING_PROVIDERS
    parser = argparse.ArgumentParser(description='OMSTD Example')
    parser.add_argument("-m", "--md5", dest="password", help="MD5 hashed value to test",
                        required=True)
    parser.add_argument("-v", dest="verbosity", help="verbosity level")
    parser.add_argument("-p", dest="provider", help="select provider used to get cracked MD5 hash. Defaul: all.",
                        default="all")
    parser.add_argument("--proxy", dest="proxy", help="proxy in format: http://USER:PASS@IP:PORT")
    parser.add_argument("--list-providers", dest="list_providers", action="store_true", help="list password providers",
                        default=False)

    params = parser.parse_args()

    # List providers?
    if params.list_providers is True:
        print("\nCracking providers:")
        print("  - all (special case. Select all providers)")
        for x in PASSWORD_MD5_CRACKING_PROVIDERS:
            print("  - %s" % x)
        print()
        exit()

    # Set config
    try:
        input_parameters = Parameters(md5_hash=params.password,
                                      verbosity=params.verbosity,
                                      provider=params.provider,
                                      proxy=params.proxy)
    except ValueError as e:
        print(e)
        exit()

    run_console(input_parameters)


# if __name__ == '__main__':
if __name__ == "__main__" and __package__ is None:
    import sys
    import os
    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.insert(1, parent_dir)
    import omstd_ch_001
    __package__ = str("omstd_ch_001")
    del sys, os

    main()


# -*- coding: utf-8 -*-

"""
Project name: hh-001
Project URL: https://github.com/cr0hn/the_open_shortener

Copyright (c) 2014, cr0hn<-AT->cr0hn.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

__author__ = 'cr0hn - cr0hn<-at->cr0hn.com (@ggdaniel)'


import json
import urllib.request

from random import randint

from .data import Parameters, PASSWORD_MD5_CRACKING_PROVIDERS, InvalidHashFormat, HashNotFound


# ----------------------------------------------------------------------
def check_md5(config):
    """
    Find MD5 hash in providers.

    :param config: Parameters object
    :type config: Parameters

    :return: plain string with text or exception if not found hash
    :rtype: str

    :raises: HashNotFound, InvalidHashFormat
    """
    if not isinstance(config, Parameters):
        raise TypeError("Expected Parameters, got '%s' instead" % type(config))

    providers_to_check = PASSWORD_MD5_CRACKING_PROVIDERS if config.provider == "all" else [config.provider]

    plain_text = None
    for p in providers_to_check:
        # Make URL using md5cracker API
        url = "http://md5cracker.org/api/api.cracker.php?r=%s&database=%s&hash=%s" % (
            randint(500, 10000),
            p,
            config.md5_hash)

        # Proxy setted?
        open_fn = None
        if config.proxy is not None:
            proxy_handler = urllib.request.ProxyHandler({config.proxy.scheme: config.proxy.netloc})

            if config.proxy_user is not None:
                proxy_auth_handler = urllib.request.ProxyBasicAuthHandler()
                proxy_auth_handler.add_password('realm', 'host', config.proxy_user, config.proxy_pass)

                opener = urllib.request.build_opener(proxy_handler, proxy_auth_handler)

            opener = urllib.request.build_opener(proxy_handler)
            # This time, rather than install the OpenerDirector, we use it directly:
            open_fn = opener.open
        else:
            open_fn = urllib.request.urlopen

        # Get remote info
        u = open_fn(url)
        _tmp_results = u.read().decode('utf-8')

        if _tmp_results is None:
            continue

        _json_results = json.loads(_tmp_results)

        # Its fails?
        if _json_results['status'] is False:
            # Check if reason is for not recoverable error
            if 'Invalid hash' in _json_results['message']:
                raise InvalidHashFormat("Invalid Hash Format")

            # It not found hash continue
            continue

        else:
            # Hash found!!!!
            plain_text = _json_results['result']
            break

    if plain_text is None:
        HashNotFound("Plain text not found for hash: '%s'" % config.md5_hash)

    return plain_text


# -*- coding: utf-8 -*-

"""
Project name: Open Methodology for Security Tool Developers
Project URL: https://github.com/cr0hn/OMSTD

Copyright (c) 2014, cr0hn<-AT->cr0hn.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""


"""
API file
"""

__author__ = 'cr0hn - cr0hn<-at->cr0hn.com (@ggdaniel)'

__all__ = ["check_md5", "run_check_md5_hash"]

# Import data
from .lib.data import *
from .lib.check_md5 import check_md5


# ----------------------------------------------------------------------
def run_check_md5_hash(input_parameters):
    """
    Checks MD5 hash and return an Results object.

    :param input_parameters: Parameters object with config
    :type input_parameters: Parameters

    :return: Results object
    :rtype: Results

    :raises: TypeError
    """
    if not isinstance(input_parameters, Parameters):
        raise TypeError("Expected Parameters, got '%s' instead" % type(input_parameters))

    try:
        r = check_md5(input_parameters)

        return Results(plain_password=r,
                       is_hash_found=True)
    except HashNotFound as e:

        return Results(plain_password=None,
                       is_hash_found=False)


# ----------------------------------------------------------------------
def run_console(input_parameters):
    """
    Run for command line interface. It's make all steps of tool:
    - Looking find
    - Get password
    - Display into screen

    :param input_parameters: Parameters object with config
    :type input_parameters: Parameters

    :raises: TypeError
    """
    try:
        result = run_check_md5_hash(input_parameters)

        # Display plain text
        print("[**] Plain text FOUND!!!. Decoded password is ----> %s <----." % result.plain_password)
    except InvalidHashFormat as e:
        print("\n[!] An error was found while try to get password: %s\n" % e)
    except HashNotFound as e:
        print("\n[-] Plain text not found for hash '%s'" % input_parameters.md5_hash)


from distutils.core import setup
setup(
    name='OMSTD-ch-001',
    version='0.1',
    packages=['omstd_ch_001', 'omstd_ch_001.lib'],
    url='http://omstd.readthedocs.org/cracking/ch_001.html',
    license='BSD',
    entry_points={
        'console_scripts': [
            'omstd-ch-001 = omstd_ch_001.ch_001:main',
            ],
    },
    author='cr0hn',
    author_email='cr0hn<-at->cr0hn.com',
    description='OMSTD CH-001 study case: cracking MD5 password using online resolver services.',
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        ],
)

